/**
  * gowiny-vue-class v1.0.61
  * https://gitee.com/gowiny/vue-class
  *
  * (c) 2022-present gowiny
  * @license MIT
  *
  * Date: 2022-03-26T05:18:37Z
  */
var GowinyVueClass = (function (exports, jsUtils, vue) {
    'use strict';

    const EXCLUDE_SUPER_PROPS = ['data', 'setup'];
    function defineProxy(proxy, key, target) {
        Object.defineProperty(proxy, key, {
            get: () => target[key].value,
            set: (value) => {
                target[key].value = value;
            },
            enumerable: true,
            configurable: true,
        });
    }
    function getSuper(Ctor) {
        const superProto = Object.getPrototypeOf(Ctor.prototype);
        if (!superProto) {
            return undefined;
        }
        return superProto.constructor;
    }
    function getOwn(value, key) {
        return value.hasOwnProperty(key) ? value[key] : undefined;
    }
    function copyNotExistProps(src, dest, propNames) {
        let result;
        if (src) {
            propNames = propNames || Object.getOwnPropertyNames(src);
            if (propNames.length > 0) {
                result = {};
                propNames.forEach(item => {
                    const srcVal = src[item];
                    if (!jsUtils.isNull(srcVal) && jsUtils.isNull(dest[item])) {
                        result[item] = srcVal;
                    }
                });
            }
            result = result || { ...src };
        }
        return result;
    }
    const defaultBeforeFactoryHandle = function (ctx) {
        const options = ctx.options;
        const Ctor = ctx.classType;
        const proto = Ctor.prototype;
        const propNames = Object.getOwnPropertyNames(proto);
        // Inject base options as a mixin
        const base = getOwn(Ctor, '__b');
        if (base) {
            options.mixins = options.mixins || [];
            options.mixins.unshift(base);
        }
        options.methods = { ...options.methods };
        options.computed = { ...options.computed };
        propNames.forEach((key) => {
            if (key === 'constructor') {
                return;
            }
            // hooks
            if (Ctor.__h.indexOf(key) > -1) {
                options[key] = proto[key];
                return;
            }
            const descriptor = Object.getOwnPropertyDescriptor(proto, key);
            // methods
            if (typeof descriptor.value === 'function') {
                options.methods[key] = descriptor.value;
                return;
            }
            // computed properties
            if (descriptor.get || descriptor.set) {
                options.computed[key] = {
                    get: descriptor.get,
                    set: descriptor.set,
                };
                return;
            }
        });
        // from Vue Loader
        const injections = [
            'render',
            'ssrRender',
            '__file',
            '__cssModules',
            '__scopeId',
            '__hmrId',
        ];
        injections.forEach((key) => {
            if (Ctor[key]) {
                options[key] = Ctor[key];
            }
        });
        // Handle super class options
        const Super = getSuper(Ctor);
        if (Super) {
            const superOptions = Super.__vccOpts;
            const extendOptions = copyNotExistProps(superOptions, options, Ctor.__h);
            if (extendOptions.watch && options.watch) {
                extendOptions.watch = copyNotExistProps(extendOptions.watch, options.watch);
            }
            EXCLUDE_SUPER_PROPS.forEach(item => {
                delete extendOptions[item];
            });
            options.extends = extendOptions;
        }
        return options;
    };
    const defaultDataFactory = function (ctx) {
        const result = function () {
            const Ctor = ctx.classType;
            const excludeProps = ctx.excludeProps;
            const data = new Ctor();
            const plainData = {};
            const dataKeys = Object.keys(data);
            dataKeys.forEach((key) => {
                if (data[key] === undefined || (data[key] && data[key].__s) ||
                    excludeProps.indexOf(key) > -1) {
                    return;
                }
                plainData[key] = vue.ref(data[key]);
                defineProxy(data, key, plainData);
            });
            return plainData;
        };
        return result;
    };
    const defaultAfterFactoryHandle = function (ctx) {
        const options = ctx.options;
        const excludeKeys = [...ctx.excludeKeys];
        if (options.computed) {
            excludeKeys.push(...Object.keys(options.computed));
        }
        if (options.props) {
            excludeKeys.push(...Object.keys(options.props));
        }
        const excludeMethods = [...excludeKeys, ...ctx.excludeMethods];
        const excludeProps = [...excludeKeys, ...ctx.excludeProps];
        excludeMethods.forEach((key) => {
            delete options.methods[key];
        });
        ctx.excludeProps = excludeProps;
        return options;
    };
    class OptionsContextImpl {
        options;
        classType;
        dataFactory;
        beforeHandle;
        afterHandle;
        attrs = new Map();
        factoryMap = {};
        excludeKeys = [];
        excludeMethods = [];
        excludeProps = [];
        constructor(options, target) {
            this.options = options;
            this.classType = target;
            this.dataFactory = defaultDataFactory;
            this.beforeHandle = defaultBeforeFactoryHandle;
            this.afterHandle = defaultAfterFactoryHandle;
        }
        addFactory(factory) {
            this.factoryMap[factory.name] = factory;
        }
        removeFactory(name) {
            delete this.factoryMap[name];
        }
        hasFactory(name) {
            return Reflect.has(this.factoryMap, name);
        }
        getOrderedFactories() {
            let list = Object.values(this.factoryMap);
            list = list.sort((o1, o2) => {
                return o1.order - o2.order;
            });
            return list;
        }
        clearFactories() {
            const keys = Object.keys(this.factoryMap);
            keys.forEach(key => {
                delete this.factoryMap[key];
            });
        }
        getAttr(name) {
            return this.attrs.get(name);
        }
        hasAttr(name) {
            return this.attrs.has(name);
        }
        setAttr(name, value) {
            this.attrs.set(name, value);
        }
        removeAttr(name) {
            this.attrs.delete(name);
        }
        clearAttrs() {
            this.attrs.clear();
        }
    }
    class VueImpl {
        static __h = [
            'beforeCreate',
            'created',
            'beforeMount',
            'mounted',
            'beforeUnmount',
            'unmounted',
            'beforeUpdate',
            'updated',
            'activated',
            'deactivated',
            'render',
            'errorCaptured',
            'serverPrefetch',
        ];
        static get __vccOpts() {
            // Early return if `this` is base class as it does not have any options
            if (this === Vue) {
                return {};
            }
            const Ctor = this;
            const cache = getOwn(Ctor, '__c');
            if (cache) {
                return cache;
            }
            // If the options are provided via decorator use it as a base
            let options = { ...getOwn(Ctor, '__o') };
            const optionsContext = new OptionsContextImpl(options, Ctor);
            const decorators = getOwn(Ctor, '__d');
            if (decorators) {
                decorators.forEach((fn) => fn(optionsContext));
            }
            if (optionsContext.beforeHandle) {
                options = optionsContext.beforeHandle(optionsContext);
                optionsContext.options = options;
            }
            const factories = optionsContext.getOrderedFactories();
            for (let i = 0; i < factories.length; i++) {
                const item = factories[i];
                options = item.handle(optionsContext);
                optionsContext.options = options;
            }
            if (optionsContext.afterHandle) {
                options = optionsContext.afterHandle(optionsContext);
                optionsContext.options = options;
            }
            const dataFactory = optionsContext.dataFactory || defaultDataFactory;
            options.data = dataFactory(optionsContext);
            Ctor.__c = options;
            return options;
        }
        static registerHooks(keys) {
            this.__h.push(...keys);
        }
        static with(Props) {
            const propsMeta = new Props();
            const props = {};
            Object.keys(propsMeta).forEach((key) => {
                const meta = propsMeta[key];
                props[key] = meta ?? null;
            });
            class PropsMixin extends this {
                static __b = {
                    props,
                };
            }
            return PropsMixin;
        }
    }
    const Vue = VueImpl;

    var vuePlugin = {
        install: (app, options) => {
            app.config.unwrapInjectedRef = true;
            const hooks = options.hooks;
            if (hooks) {
                Vue.registerHooks(hooks);
            }
        }
    };

    function Options(options) {
        return (Component) => {
            Component.__o = options;
            return Component;
        };
    }

    function getOptionsByClass(Ctor) {
        let options;
        if (Ctor.__vccOpts) {
            options = Ctor.__vccOpts;
        }
        else {
            options = Ctor;
        }
        //console.log('getOptionsByClass Ctor',Ctor)
        //console.log('getOptionsByClass options',options)
        return options;
    }
    function mergeSetup(...args) {
        if (!args) {
            return;
        }
        else if (args.length == 1) {
            return args[0];
        }
        else {
            const result = {};
            args.forEach((item) => {
                if (!item) {
                    return;
                }
                else if (jsUtils.isFunction(item)) {
                    result.render = item;
                }
                else {
                    Object.assign(result, item);
                }
            });
            return result;
        }
    }
    function createDecorator(factory) {
        return (target, key, index) => {
            const Ctor = typeof target === 'function'
                ? target
                : target.constructor;
            if (!Ctor.__d) {
                Ctor.__d = [];
                Ctor.__d.__ctor = Ctor;
            }
            else if (Ctor.__d.__ctor !== Ctor) {
                Ctor.__d = [].concat(Ctor.__d);
                Ctor.__d.__ctor = Ctor;
            }
            if (typeof index !== 'number') {
                index = undefined;
            }
            Ctor.__d.push((ctx) => factory(ctx, key, index));
        };
    }
    function mixins(...Ctors) {
        return class MixedVue extends Vue {
            static __b = {
                mixins: Ctors.map((Ctor) => Ctor.__vccOpts),
            };
            constructor(...args) {
                super(...args);
                Ctors.forEach((Ctor) => {
                    const data = new Ctor(...args);
                    Object.keys(data).forEach((key) => {
                        this[key] = data[key];
                    });
                });
            }
        };
    }

    function create$7(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            if (!options.watch) {
                options.watch = {};
            }
            const name = params || key;
            const watch = options.watch;
            watch[name] = descriptor?.value;
            ctx.excludeKeys.push(key);
        })(target, propertyKey);
    }
    function Watch(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create$7(propertyKey, target, propertyKey, descriptor);
        }
        else {
            const params = target;
            return function (target, propertyKey, descriptor) {
                create$7(params, target, propertyKey, descriptor);
            };
        }
    }

    function create$6(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            if (!options.props) {
                options.props = {};
            }
            const props = options.props;
            const oldDefault = params.default;
            const defType = typeof (oldDefault);
            if (descriptor) { //定义在方法上
                if (defType === 'undefined') {
                    params.default = function () { return descriptor.value; };
                }
            }
            else { //定义在属性上
                if (defType === 'object') {
                    params.default = jsUtils.clone(oldDefault);
                }
                else if (defType === 'undefined') {
                    params.default = jsUtils.getClassDefaultValue(ctx.classType, key);
                }
            }
            props[key] = params;
        })(target, propertyKey);
    }
    function Prop(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create$6({}, target, propertyKey, descriptor);
        }
        else {
            const params = target;
            return function (target, propertyKey, descriptor) {
                create$6(params, target, propertyKey, descriptor);
            };
        }
    }

    function create$5(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            options.methods = options.methods || {};
            const name = params || key;
            options.methods[key] = function (...args) {
                return this.$store.dispatch(name, ...args);
            };
        })(target, propertyKey);
    }
    function Action(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create$5(propertyKey, target, propertyKey);
        }
        else {
            const params = target;
            return function (target, propertyKey, descriptor) {
                create$5(params, target, propertyKey);
            };
        }
    }

    function create$4(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            options.computed = options.computed || {};
            const name = params || key;
            options.computed[key] = function () {
                const getter = this.$store.getters[name];
                return getter;
            };
        })(target, propertyKey);
    }
    function Getter(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create$4(propertyKey, target, propertyKey);
        }
        else {
            const params = target;
            return function (target, propertyKey, descriptor) {
                create$4(params, target, propertyKey);
            };
        }
    }

    function create$3(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            options.computed = options.computed || {};
            const arr = params ? params.split(/[/.]/) : undefined;
            options.computed[key] = function () {
                let data = this.$store.state;
                if (!arr) {
                    return data[key];
                }
                else {
                    arr.forEach((item) => {
                        data = data[item];
                    });
                    return data;
                }
            };
        })(target, propertyKey);
    }
    function State(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create$3(propertyKey, target, propertyKey);
        }
        else {
            const params = target;
            return function (target, propertyKey, descriptor) {
                create$3(params, target, propertyKey);
            };
        }
    }

    function create$2(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            options.methods = options.methods || {};
            const dname = params || key;
            options.methods[key] = function (...args) {
                return this.$store.commit(dname, ...args);
            };
        })(target, propertyKey);
    }
    function Mutation(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create$2(propertyKey, target, propertyKey);
        }
        else {
            const params = target;
            return function (target, propertyKey, descriptor) {
                create$2(params, target, propertyKey);
            };
        }
    }

    function create$1(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            if (!options.inject) {
                options.inject = {};
            }
            const data = Object.assign({ from: key }, params);
            if (jsUtils.isNull(data.default)) {
                if (descriptor?.value) {
                    data.default = function () { return descriptor.value; };
                }
                else {
                    data.default = jsUtils.getClassDefaultValue(ctx.classType, key);
                }
            }
            const inject = options.inject;
            inject[key] = data;
            ctx.excludeKeys.push(key);
        })(target, propertyKey);
    }
    function Inject(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create$1({ from: propertyKey }, target, propertyKey, descriptor);
        }
        else {
            const targetType = typeof target;
            const params = targetType === 'string' || targetType === 'symbol' ? { from: target } : target;
            return function (target, propertyKey, descriptor) {
                create$1(params, target, propertyKey, descriptor);
            };
        }
    }

    function create(params, target, propertyKey, descriptor) {
        createDecorator(function (ctx, key) {
            const options = ctx.options;
            if (!options.provide) {
                const provideFun = function () {
                    const result = Object.create(null);
                    for (const [key, params] of provideFun.values) {
                        //默认响应式设置
                        if (params.ref === false) {
                            result[params.name] = this[key];
                        }
                        else {
                            result[params.name] = vue.computed(() => this[key]);
                        }
                    }
                    for (const [key, params] of provideFun.funs) {
                        result[params.name] = (...args) => {
                            return this[key](...args);
                        };
                    }
                    return result;
                };
                provideFun.values = new Map();
                provideFun.funs = new Map();
                options.provide = provideFun;
            }
            const provide = options.provide;
            if (descriptor?.value) {
                provide.funs.set(key, params);
            }
            else if (descriptor?.get) {
                provide.values.set(key, params);
            }
            else {
                provide.values.set(key, params);
            }
        })(target, propertyKey);
    }
    function Provide(target, propertyKey = "", descriptor) {
        if (propertyKey) {
            create({ name: propertyKey }, target, propertyKey, descriptor);
        }
        else {
            const targetType = typeof target;
            const params = targetType === 'string' || targetType === 'symbol' ? { name: target } : target;
            return function (target, propertyKey, descriptor) {
                create(params, target, propertyKey, descriptor);
            };
        }
    }

    exports.Action = Action;
    exports.Getter = Getter;
    exports.Inject = Inject;
    exports.Mutation = Mutation;
    exports.Options = Options;
    exports.Prop = Prop;
    exports.Provide = Provide;
    exports.State = State;
    exports.Vue = Vue;
    exports.Watch = Watch;
    exports["default"] = vuePlugin;
    exports.getOptionsByClass = getOptionsByClass;
    exports.mergeSetup = mergeSetup;
    exports.mixins = mixins;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, GowinyUtils, Vue);
